### Install packages and load libraries ####

# install.packages(c("tidyverse", "lme4", "gridExtra", "geomorph", "MASS", "scales", "RColorBrewer")) # uncomment this line if you need to install or update any of the packages necessary to run this script

library(tidyverse)
library(lme4)
library(gridExtra)
library(geomorph)
library(MASS)
library(scales)
library(RColorBrewer)

options(scipen=999)

#### set overall color palette ####
palette_5 <- brewer.pal(5, "Set1")
palette_2 <- palette_5[c(2,4)]
palette_4 <- palette_5[c(1,2,4,5)]

#### Create manuscript Figure 2 ####
# Plot mean shape differences between landmark datasets 

# Read in original tps files of whichever two landmark trials you wish to compare
# In this example, interobserver error is plotted between the new observer and

# experienced observer's second trials (Fig 2C of the manuscript)

data1<- readland.tps("ProcANOVA/Dinolite_NoTilt_EO_T2.tps", specID = "imageID") 
data2<- readland.tps("ProcANOVA/Dinolite_NoTilt_NO_T2.tps", specID = "imageID") 

consensus_ref <- mshape(data1) # calculate mean shape of the reference dataset
consensus_target <- mshape(data2) # calculate mean shape of the target comparative dataset

plotRefToTarget(consensus_ref,consensus_target, method = "TPS", # LM displacement can be visualized as "TPS", "points", or "vector"
  gridPars=gridPar(pt.bg = "green", pt.size = 1), mag=1) # increase "mag" to magnify shape deformation


#boxplot of LDA classification error by observer and camera (Figure 3A) ####

#(Data from Table 1 of the manuscript)
observer.error <- read.csv("Figure_data/Input/Observer_LDA_error.csv", row.names = 1)

  BoxPlot<-ggplot(observer.error, aes(x = Source, y = Percent.PGM.error, color = Device)) +
  theme_light() +
  geom_boxplot(notch = FALSE, fill='#A4A4A4') +
  scale_color_manual(values = palette_2)
  BoxPlot

#### LDA Plots of Observer Means (Figure 3B, C) ####

### Read in TPS files
d1 <- readland.tps("LDA/Dinolite_NoTilt_EO_T1_Dep1.tps", specID = "imageID")
d2 <- readland.tps("LDA/Dinolite_NoTilt_EO_T2_Dep1.tps", specID = "imageID")
d3 <- readland.tps("LDA/Dinolite_NoTilt_NO_T1_Dep1.tps", specID = "imageID")
d4 <- readland.tps("LDA/Dinolite_NoTilt_NO_T2_Dep1.tps", specID = "imageID")
d5 <- readland.tps("LDA/Dinolite_Tilted_EO_T1_Dep1.tps", specID = "imageID")
### Perform GPA ###
d1.gpa <-gpagen(d1)
d2.gpa <-gpagen(d2)
d3.gpa <- gpagen(d3)
d4.gpa <- gpagen(d4) 
d5.gpa <- gpagen(d5)
# Convert LM coords from gpa files to 2D arrays 
d1_coords <- two.d.array(d1.gpa$coords)
d2_coords <- two.d.array(d2.gpa$coords)
d3_coords <- two.d.array(d3.gpa$coords)
d4_coords <- two.d.array(d4.gpa$coords)
d5_coords <- two.d.array(d5.gpa$coords)
#Compute Means
EO_Dinolite_Mean <- (d1_coords+d2_coords)/2
NO_Dinolite_Mean <- (d3_coords+d4_coords)/2
# assign species-group classifier
classifier_dino <- read.csv("LDA/Dinolite_Dep1_classifier.csv", header=T)

# combine landmark coordinates with species-group classifiers
EO_Dinolite_class <- cbind(classifier_dino, EO_Dinolite_Mean) 
NO_Dinolite_class <- cbind(classifier_dino, NO_Dinolite_Mean)
EO_Tilted_class <- cbind(classifier_dino, d5_coords)

# save extant Microtus LDA training set (n = 247)
write.csv(EO_Dinolite_class[1:247,], 'Figure_data/Input/EO_Dinolite_Mean.csv') 
write.csv(NO_Dinolite_class[1:247,], 'Figure_data/Input/NO_Dinolite_Mean.csv') 
write.csv(EO_Tilted_class[1:247,], 'Figure_data/Input/EO_Dinolite_Tilted.csv')

##LDA
DiscF<-read.table('Figure_data/Input/EO_Dinolite_Mean.csv', header=TRUE, sep=",") 
known<-DiscF[,3:44]# assign predictor variables (i.e., the 42 x, y landmark coordinates)
ids<-DiscF[,2] # assign grouping variables (i.e., the five Microtus species)


train<-lda(known,ids) # create LDA training functions
train_cv<-lda(known,ids, CV=T) # Include leave-one-out cross-validation
train
plda <- predict(object = train,
                newdata = known)

# plot polygons for each species
dataset = data.frame(species = DiscF$species, lda = plda$x[,1:2])
prop.lda = train$svd^2/sum(train$svd^2)
# create polygon hulls
hull_species <- dataset %>%
  group_by(species) %>%
  slice(chull(lda.LD1, lda.LD2))

Figure <- ggplot(dataset, aes(lda.LD1, lda.LD2)) + 
  geom_point(shape = 20) +
  labs(x = paste("LD1 (", percent(prop.lda[1]), ")", sep=""),
       y = paste("LD2 (", percent(prop.lda[2]), ")", sep="")) +
  theme_light()
Figure + aes(fill = factor(species)) + 
  geom_polygon(data = hull_species, alpha = 0.5)+
  scale_fill_manual(values = palette_5, name = "Species", labels = c("M. californicus", "M. longicaudus", "M. montanus", "M. oregoni", "M. townsendii"))

# CV classification results:
CVclassTable<-table(train_cv$class, DiscF[,2])
CVclassTable
CVerror = sum(CVclassTable[row(CVclassTable) != col(CVclassTable)]) / sum(CVclassTable)*100
CVerror 


#### Linear mixed model of R2 error and extant/fossil Microtus PGM differences #### 
#(Figure 4), (Data from Table 1 of the manuscript) 
error.data <- read.csv("Figure_data/Input/Pairwise.csv")

# Fig 4A
model_A <- lmer(PGM.error ~ 1+ ProcANOVA.R2 | Error.Source, data=error.data) 
ss <- getME(model_A,c("theta","fixef"))
model_A2 <- update(model_A,start=ss)

summary(model_A2)
error.data.A <- error.data
error.data.A$fit <- predict(model_A2)  
#Add model fits to dataframe
summary(model_A)
error.data.A <- error.data
error.data.A$fit <- predict(model_A)

  ggplot(error.data.A, aes(x = ProcANOVA.R2, y = PGM.error)) +
  theme_light() +
  geom_line(aes(y=fit, lty=Error.Source), size=0.8)+
  geom_point(aes(color = Error.Source, fill = Error.Source, shape = Error.Source)) +
  scale_color_manual(values = palette_4)+
  scale_fill_manual(values = palette_4)

# Fig 4B
model_B <- lmer(Fossil.PGM ~ 1+ ProcANOVA.R2 | Error.Source, data=error.data) 
ss <- getME(model_B,c("theta","fixef"))
model_B2 <- update(model_B,start=ss)

summary(model_B2)
error.data.B <- error.data
error.data.B$fit <- predict(model_B2)   

  ggplot(error.data.B, aes(x = ProcANOVA.R2, y = Fossil.PGM)) +
  theme_light() +
  geom_line(aes(y=fit, lty=Error.Source), size=0.8)+
  geom_point(aes(color = Error.Source, fill = Error.Source, shape = Error.Source)) +
  scale_color_manual(values = palette_4)+
  scale_fill_manual(values = palette_4)
