### Install packages and load libraries ####

# install.packages(c("MASS", "tidyverse", "scales", "RColorBrewer")) # uncomment this line if you need to install or update any of the packages necessary to run this script

library(MASS)
library(tidyverse)
library(scales)
library(RColorBrewer)
options(scipen=999)

### Linear Discriminant Analysis ###

DiscF<-read.table('LDA/Output/Dinolite_NoTilt_EO_T2.csv', header=TRUE, sep=",") 
# enter the desired landmark coordinate training set from the "LDA/Output" folder.

known<-DiscF[,3:44] # assign predictor variables (i.e., the 42 x, y landmark coordinates)
ids<-DiscF[,2] # assign grouping variables (i.e., the five extant Microtus species)
# Mc = Microtus californicus, Ml = Microtus longicaudus, Mm = Microtus montanus, Mo = Microtus oregoni, 
# Mt = Microtus townsendi

train<-lda(known,ids) # create LDA training functions
train_cv<-lda(known,ids, CV=T) # Include leave-one-out cross-validation
train
plda <- predict(object = train,
                newdata = known)
# Plot species-groups along the first two LDA functions. The relative morphological distance (separation)
# among recent species-groups is a good indicator of the predicted group membership accuracy of the 
# analysis overall (e.g., manuscript Figure 3)


dataset = data.frame(species = DiscF$species, lda = plda$x[,1:2])
prop.lda = train$svd^2/sum(train$svd^2)

#set overall color palette 
palette_5 <- brewer.pal(5, "Set1")
palette_2 <- palette_5[c(2,4)]
palette_4 <- palette_5[c(1,2,4,5)]

# create polygon hulls
hull_species <- dataset %>%
  group_by(species) %>%
  slice(chull(lda.LD1, lda.LD2))

p <- ggplot(dataset, aes(lda.LD1, lda.LD2)) + 
  geom_point(shape = 20) +
  labs(x = paste("LD1 (", percent(prop.lda[1]), ")", sep=""),
       y = paste("LD2 (", percent(prop.lda[2]), ")", sep="")) +
  theme_light()

p + aes(fill = factor(species)) + 
  geom_polygon(data = hull_species, alpha = 0.5)+
  scale_fill_discrete(name = "Species", labels = c("M. californicus", "M. longicaudus", "M. montanus", "M. oregoni", "M. townsendii"))

# Training-group classification results:
class_stats<-predict(train, newdata=DiscF[,3:44])$class
classTable <- table(class_stats, DiscF[,2])
classTable
# This table show the number of individuals per species assigned to each species group
# Columns show the number of individuals of each species and rows indicate the predicted species affiliation of each specimen based on their landmark configurations
# In this example, the first column shows that 47 of the 49 individuals of 
# M. californicus are classified correctly as M. californicus (Mc, row 1) 
# and the remaining two are misclassified as M. longicaudus (Ml, row 2)

error = sum(classTable[row(classTable) != col(classTable)]) / sum(classTable)*100
error
# calculates the overall error of the training set (i.e., the total percentage of incorrectly
# classified specimens across all species groups)

# Training-group classification results using leave-one-out cross-validation
CVclassTable<-table(train_cv$class, DiscF[,2])
CVclassTable
CVerror = sum(CVclassTable[row(CVclassTable) != col(CVclassTable)]) / sum(CVclassTable)*100
CVerror # cross-validated classification error


## Predict species-group memberships of unknown fossils from Rancho La Brea, Project 23, Deposit 1

fossils<-read.table("LDA/Output/Dinolite_NoTilt_EO_T2_Dep1.csv", header=TRUE, sep=",") 
# enter the fossil landmark coordinates that have been gpa-transformed with the training set 
# above in "LDA_data_prep.R". I.e., this file name should match the one above exactly 
# but with a "_Dep1" suffix.

known<-DiscF[,3:44] # assign training variables                                  
unknown<-fossils[,3:44] # assign unknown variables 
Fossil_IDs<-predict(train, unknown) 
Fossil_IDs # classify unknowns

Fossil_IDs$class
# "$class" lists the predicted species affinity of each unknown specimen in sequential order
# In this example, 24 of the 31 fossil specimens are assigned to M. californicus (Mc), 
# five are assigned to M. longicaudus (Ml), and two are assigned to M. oregoni (Mo) 


Fossil_IDs$posterior
# "$posterior" lists the 0-1 probability of each specimen belonging to each extant species-group. 
# For example, row one lists species-group membership probabilities for the first fossil specimen 
# in our unknown dataset (LACMP23-04397) classified as "M. longicaudus" in this training set. Here, 
# the analysis is ~ 0.85 (85%) confident that LACMP23-04397 has a M. longicaudus affinity

Fossil_IDs$x
#"$x" lists the discriminant function values of each unknown specimen 

